from unittest import TestCase
import click
from flask import Flask
from ...utils import (
    response_not_found,
    Response
    )
from ...decorators import (
    is_enabled,
    command_enabled,
    route_enabled,
    use_case_enabled
)


class TestIsEnabled(TestCase):

    def test_enable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_HI': True}

        with self._app.app_context():
            @is_enabled(response_not_found, 'ENV_HI')
            def hi(name):
                return {'data': name, 'message': 'ok'}

            result = hi('success')

            self.assertEqual(result, {'data': 'success', 'message': 'ok'})
            self.assertEqual(hi.__name__, 'hi')

    def test_disable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_HI': False}

        with self._app.app_context():
            @is_enabled(response_not_found, 'ENV_HI')
            def hi(name):
                pass

            result = hi('success')

            self.assertEqual(result, ({'message': 'NOT_FOUND'}, 404))
            self.assertEqual(hi.__name__, 'hi')


class TestCommandEnabled(TestCase):
    def test_enable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_RUN_COMMAND': True}

        with self._app.app_context():

            @command_enabled('ENV_RUN_COMMAND')
            def run_command():
                click.echo('I am command')

            result = run_command()

            self.assertIsNone(result)
            self.assertEqual(run_command.__name__, 'run_command')


class TestRouteEnabled(TestCase):
    def test_enable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_RUN_ROUTE': True}

        with self._app.app_context():

            @route_enabled('ENV_RUN_ROUTE')
            def run_route():
                return dict(message='OK'), 200

            result = run_route()

            self.assertEqual(result, ({'message': 'OK'}, 200))
            self.assertEqual(run_route.__name__, 'run_route')

    def test_disable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_RUN_ROUTE': False}

        with self._app.app_context():

            @route_enabled('ENV_RUN_ROUTE')
            def run_route():
                pass

            result = run_route()
            self.assertEqual(result, ({'message': 'NOT_FOUND'}, 404))
            self.assertEqual(run_route.__name__, 'run_route')


class TestUseCaseEnabled(TestCase):
    def test_enable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_RUN_USE_CASE': True}

        with self._app.app_context():

            @use_case_enabled('ENV_RUN_USE_CASE')
            def run_use_case():
                return Response(
                    http_code=200,
                    message='OK'
                )

            result = run_use_case()

            self.assertIsInstance(result, Response)
            self.assertEqual(result.http_code, 200)
            self.assertEqual(run_use_case.__name__, 'run_use_case')

    def test_disable(self):
        self._app = Flask(__name__)
        self._app.config['FEATURE_FLAGS'] = {'ENV_RUN_USE_CASE': False}

        with self._app.app_context():

            @use_case_enabled('ENV_RUN_USE_CASE')
            def run_use_case():
                pass

            result = run_use_case()

            self.assertIsInstance(result, Response)
            self.assertEqual(result.http_code, 404)
            self.assertEqual(run_use_case.__name__, 'run_use_case')



from flask import current_app as app
from .utils import recursive_get


def flag_on(key: str) -> bool:
    """Enable or disable a feature

    Example:
    ::
        if flag_on('MY_KEY'):
            pass

    Args:
        key (str):

    Returns:
        bool:
    """
    return recursive_get(app.config, f'FEATURE_FLAGS.{key}', False)



from .decorators import is_enabled
from .decorators import command_enabled
from .decorators import route_enabled
from .decorators import use_case_enabled
from .feature_flag import flag_on


__all__ = (
    'is_enabled',
    'command_enabled',
    'route_enabled',
    'use_case_enabled',
    'flag_on',
)



from functools import reduce
import click


class Response:
    """Response use case

    Attributes:
        http_code (int): status code, default 404
        message (str) : message string, default 'NOT_FOUND'
        errors (list)
    """

    def __init__(
        self,
        http_code: int = 404,
        message: str = 'NOT_FOUND',
        errors: list = None
    ):
        self.http_code = http_code
        self.message = message
        self.errors = errors

    def __json__(self):
        return {
            'http_code': self.http_code,
            'message': self.message,
            'errors': self.errors,
        }


def recursive_get(d: dict, keys, default=None):
    """get recursive to dict

    Args:
        d (dict)
        keys (str|list|tuple): if is str keyA.keyB => [keyA, keyB]
        default (mixed): No required
    """
    if isinstance(keys, str):
        keys = keys.split('.')
    result = reduce(lambda c, k: c.get(k, {}), keys, d)
    if default is not None and result == {}:
        return default
    return result


def response_not_found() -> tuple:
    """Function not found

    Returns:
        response (tuple): status message
    """
    return dict(message='NOT_FOUND'), 404


def response_command() -> None:
    """Function command disabled

    Returns:
        None:
    """
    click.echo('command disabled')


def response_use_case() -> Response:
    """Response use case

    Returns:
        Response:
    """
    return Response()



from unittest import TestCase
from unittest.mock import patch
from ...utils import (
    response_not_found,
    response_command,
    response_use_case,
    Response
    )


class TestResponseNotFound(TestCase):
    def test_ok(self):
        result = response_not_found()
        self.assertEqual(result, ({"message": "NOT_FOUND"}, 404))


class TestResponseCommand(TestCase):
    @patch('flask_feature_flag.utils.click.echo')
    def test_ok(self, mock_echo):
        response_command()
        mock_echo.assert_called_with('command disabled')


class TestResponseUseCase(TestCase):
    def test_ok(self):
        result = response_use_case()
        self.assertIsInstance(result, Response)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="flask_feature_flag",
    version="0.0.3b0",
    author="Terminus",
    author_email="jose.salas@zinobe.com",
    description="Flask feature flag",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/terminus-zinobe/flask-feature-flag",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'flask==1.1.1',
        'bandit',
        'pytest==5.2.1',
        'pytest-cov',
        'radon'
    ]
)



import functools
from flask import current_app as app
from .utils import (
    response_command,
    response_not_found,
    response_use_case,
    recursive_get
    )


def is_enabled(response, feature):
    """Decorator to enable or disable a feature

    Args:
        response: function that returns object to return
        feature: environment variable name

    Returns:
        response: decorated function or function error
    """
    def _decorator(func):
        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            feature_name = recursive_get(
                app.config.get('FEATURE_FLAGS'),
                feature,
                default=False
            )
            if feature_name:
                return func(*args, **kwargs)
            return response()
        return _wrapper
    return _decorator


def command_enabled(feature: str):
    """Decorator to enable or disable a command

    Args:
        feature: environment variable name

    Returns:
        response: decorated function or response_command
    """
    def _decorator(func):
        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            feature_name = recursive_get(
                app.config.get('FEATURE_FLAGS'),
                feature,
                default=False
            )
            if feature_name:
                return func(*args, **kwargs)
            return response_command()
        return _wrapper
    return _decorator


def route_enabled(feature: str):
    """Decorator to enable or disable a route

    Args:
        feature: environment variable name

    Returns:
        response: decorated function or response_not_found
    """
    def _decorator(func):
        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            feature_name = recursive_get(
                app.config.get('FEATURE_FLAGS'),
                feature,
                default=False
            )
            if feature_name:
                return func(*args, **kwargs)
            return response_not_found()
        return _wrapper
    return _decorator


def use_case_enabled(feature: str):
    """Decorator to enable or disable a use case

    Args:
        feature: environment variable name

    Returns:
        response: decorated function or response_use_case
    """
    def _decorator(func):
        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            feature_name = recursive_get(
                app.config.get('FEATURE_FLAGS'),
                feature,
                default=False
            )
            if feature_name:
                return func(*args, **kwargs)
            return response_use_case()
        return _wrapper
    return _decorator
